import logging
import os

import cv2
import uvicorn
from fastapi import FastAPI, Form, Request
from fastapi.responses import HTMLResponse, StreamingResponse
from fastapi.staticfiles import StaticFiles
from fastapi.templating import Jinja2Templates
from starlette.websockets import WebSocket

from kamera.camera import Camera
from kamera.settings import AVAILABLE_FILTERS

# Initialize FastAPI app
app = FastAPI()

# Initialize camera and logger
camera = Camera()
logger = logging.getLogger("kamera")
logging.basicConfig(level=logging.INFO)

static_path = os.path.join(os.path.dirname(__file__), "static")
template_path = os.path.join(os.path.dirname(__file__), "templates")

if not os.path.exists(static_path):
    raise RuntimeError(f"Static directory not found: {static_path}")


# Serve static files
app.mount("/static", StaticFiles(directory=static_path), name="static")
templates = Jinja2Templates(directory=template_path)


@app.get("/", response_class=HTMLResponse)
async def index(request: Request):
    """Serve the web interface."""
    return templates.TemplateResponse("index.html", {"request": request})


@app.get("/video_feed")
async def video_feed():
    """Stream video frames."""

    def generate_frames():
        while True:
            frame = camera.get_frame()
            if frame is None:
                break
            yield (b"--frame\r\nContent-Type: image/jpeg\r\n\r\n" + frame + b"\r\n")

    return StreamingResponse(
        generate_frames(), media_type="multipart/x-mixed-replace; boundary=frame"
    )


@app.post("/settings")
async def update_settings(
    rotate: int = Form(0),
    brightness: float = Form(1.0),
    grayscale: bool = Form(False),
    mirror: bool = Form(False),
    filter_name: str = Form("none"),
    camera_index: int = Form(0),  # Added camera index
):
    """Update camera settings dynamically."""
    # Reinitialize camera if the index has changed
    if camera_index != camera.camera_index:
        camera.release()
        camera.camera_index = camera_index
        camera.cap = cv2.VideoCapture(camera.camera_index)
        logger.info(f"Camera index changed to {camera.camera_index}")

    camera.rotate = rotate
    camera.brightness = brightness
    camera.grayscale = grayscale
    camera.mirror = mirror
    camera.filter_name = filter_name
    logger.info("Settings updated: %s", camera.__dict__)
    return {"status": "success", "message": "Settings updated successfully"}


@app.websocket("/ws/video_feed")
async def websocket_video_feed(websocket: WebSocket):
    """Serve video feed via WebSocket."""
    await websocket.accept()
    try:
        while True:
            frame = camera.get_frame()
            if frame is None:
                break
            await websocket.send_bytes(frame)
    except Exception as e:
        logger.error("WebSocket error: %s", str(e))
    finally:
        await websocket.close()


@app.on_event("shutdown")
def shutdown_event():
    """Release camera resources on shutdown."""
    camera.release()
    logger.info("Camera released.")


def main():
    uvicorn.run("kamera.app:app", host="0.0.0.0", port=4141, reload=True)


if __name__ == "__main__":
    main()



import cv2
import numpy as np

AVAILABLE_FILTERS = ["none", "sepia", "blur", "edges"]


def adjust_brightness(frame, brightness=1.0):
    """Adjust brightness of the frame."""
    return cv2.convertScaleAbs(frame, alpha=brightness, beta=0)


def rotate_frame(frame, angle=0):
    """Rotate the frame."""
    if angle == 90:
        return cv2.rotate(frame, cv2.ROTATE_90_CLOCKWISE)
    elif angle == 180:
        return cv2.rotate(frame, cv2.ROTATE_180)
    elif angle == 270:
        return cv2.rotate(frame, cv2.ROTATE_90_COUNTERCLOCKWISE)
    return frame


def mirror_frame(frame):
    """Mirror the frame."""
    return cv2.flip(frame, 1)


def convert_to_grayscale(frame):
    """Convert the frame to grayscale."""
    return cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)


def apply_filter(frame, filter_name):
    """Apply selected filter to the frame."""
    if filter_name == "sepia":
        kernel = np.array(
            [[0.272, 0.534, 0.131], [0.349, 0.686, 0.168], [0.393, 0.769, 0.189]]
        )
        return cv2.transform(frame, kernel)
    elif filter_name == "blur":
        return cv2.GaussianBlur(frame, (15, 15), 0)
    elif filter_name == "edges":
        return cv2.Canny(frame, 100, 200)
    return frame


def process_frame(
    frame, rotate=0, brightness=1.0, grayscale=False, filter_name="none", mirror=False
):
    """Apply all frame adjustments."""
    if grayscale:
        frame = convert_to_grayscale(frame)
    if mirror:
        frame = mirror_frame(frame)
    frame = adjust_brightness(frame, brightness)
    frame = rotate_frame(frame, rotate)
    frame = apply_filter(frame, filter_name)

    return frame



import cv2

from kamera.settings import process_frame


class Camera:
    def __init__(
        self,
        camera_index=0,
        rotate=0,
        brightness=1.0,
        grayscale=False,
        filter_name="none",
        mirror=False,
    ):
        self.camera_index = camera_index
        self.rotate = rotate
        self.brightness = brightness
        self.grayscale = grayscale
        self.filter_name = filter_name
        self.mirror = mirror
        self.cap = cv2.VideoCapture(self.camera_index)

    def get_frame(self):
        success, frame = self.cap.read()
        if not success:
            return None
        frame = process_frame(
            frame,
            rotate=self.rotate,
            brightness=self.brightness,
            grayscale=self.grayscale,
            filter_name=self.filter_name,
            mirror=self.mirror,
        )
        _, buffer = cv2.imencode(".jpg", frame)
        return buffer.tobytes()

    def release(self):
        self.cap.release()



from setuptools import find_packages, setup

setup(
    name="kamera",
    version="0.2.7",
    author="Mert Cobanov",
    author_email="mertcobanov@gmail.com",
    description="A simple webcam streaming package",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/cobanov/kamera",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "fastapi",
        "uvicorn",
        "jinja2",
        "opencv-python",
        "python-multipart",
    ],
    entry_points={
        "console_scripts": [
            "kamera=kamera.app:main",
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
)
